<?php

declare(strict_types=1);

namespace Drupal\kameleoon\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Kameleoon settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'kameleoon_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['kameleoon.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['sitecode'] = [
      '#type' => 'key_select',
      '#title' => $this->t('Project sitecode'),
      '#description' => $this->t('(The project sitecode is provided by <a href="https://kameleoon.com">Kameleoon</a>).'),
      '#default_value' => $this->config('kameleoon.settings')->get('sitecode'),
    ];
    $form['script'] = [
      '#type' => 'select',
      '#title' => $this->t('Select script'),
      '#options' => [
        'async' => $this->t('Asynchronous tag without anti-flicker'),
        'async-anti-flicker' => $this->t('Asynchronous tag with anti-flicker'),
        'sync' => $this->t('Synchronous tag'),
      ],
      '#description' => $this->t('See the <a href="https://developers.kameleoon.com/web-experimentation/simple-implementation/">documentation</a>.'),
      '#default_value' => $this->config('kameleoon.settings')->get('script'),
      '#attributes' => [
        'id' => 'select-script',
      ],
    ];
    $form['loading_timeout'] = [
      '#type' => 'number',
      '#min' => 0,
      '#step' => 100,
      '#title' => $this->t('Loading timeout'),
      '#description' => $this->t('Duration in milliseconds to wait while the Kameleoon application file is loaded. The loading time is used in the script "asynchronous tag with anti-flicker".'),
      '#default_value' => $this->config('kameleoon.settings')->get('loading_timeout'),
      '#states' => [
        'visible' => [
          ':input[id="select-script"]' => ['value' => 'async-anti-flicker'],
        ],
      ],
    ];
    $form['enable_in_admin'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Kameleoon in the Drupal administration'),
      '#description' => $this->t('If checked, the Kameleoon script will also be enabled in the Drupal admin pages, user editing pages and when editing or creating content.'),
      '#default_value' => $this->config('kameleoon.settings')->get('enable_in_admin'),
    ];
    $form['enable_for_auth'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Kameleoon for authenticated users'),
      '#description' => $this->t('If checked, the Kameleoon script will be enabled also for authenticated users.'),
      '#default_value' => $this->config('kameleoon.settings')->get('enable_for_auth'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Validate, if the loading timeout is a positive integer.
    if ($form_state->getValue('loading_timeout') < 0) {
      $form_state->setErrorByName(
        'loading_timeout',
        $this->t('The loading timeout must be a positive integer.'),
      );
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('kameleoon.settings')
      ->set('sitecode', $form_state->getValue('sitecode'))
      ->set('script', $form_state->getValue('script'))
      ->set('loading_timeout', $form_state->getValue('loading_timeout'))
      ->set('enable_in_admin', $form_state->getValue('enable_in_admin'))
      ->set('enable_for_auth', $form_state->getValue('enable_for_auth'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
